"""
Module-level doctest

    >>> 1
    1
    >>> add_one(_)
    2
"""


def add_one(i):
    """
    function doctest

    >>> add_one(1)
    2
    >>> add_one(2)
    3
    >>> add_one('steve')
    joe    
    """
    return i + 1
